### A Pluto.jl notebook ###
# v0.20.4

using Markdown
using InteractiveUtils

# This Pluto notebook uses @bind for interactivity. When running this notebook outside of Pluto, the following 'mock version' of @bind gives bound variables a default value (instead of an error).
macro bind(def, element)
    #! format: off
    quote
        local iv = try Base.loaded_modules[Base.PkgId(Base.UUID("6e696c72-6542-2067-7265-42206c756150"), "AbstractPlutoDingetjes")].Bonds.initial_value catch; b -> missing; end
        local el = $(esc(element))
        global $(esc(def)) = Core.applicable(Base.get, el) ? Base.get(el) : iv(el)
        el
    end
    #! format: on
end

# ╔═╡ a145e2fe-a903-11eb-160f-df7ea83fa3e6
begin
	using HypertextLiteral, PlutoUI, PlutoPlotly
	using CSV, DataFrames, Dates, PeriodicalDates, SparseArrays, StatsBase
end

# ╔═╡ fd79048d-2700-4f42-ac1e-5cb4df35e5ea
html"""
<picture>
  <source srcset="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/negative/rgb_ibs_en_horizontal_negative.png" media="(prefers-color-scheme: dark)" width="240px!important" style="float:right;">
  <img src="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/main/rgb_ibs_en_horizontal_main.png" width="240px" style="float:right;">
</picture>
"""

# ╔═╡ 943edb36-acc8-4b2a-94ab-c544f9eb279b
md"""
# Week 7 - Aggregate Supply & the Phillips Curve

## Exercises

**Macroeconomics, ISCTE-IUL**
"""

# ╔═╡ 1245aef9-18ce-42f5-af82-c2e6cce70798
md"""
**Vivaldo Mendes, Ricardo Gouveia-Mendes, Luís Clemente-Casinhas**

**March 2025**
"""

# ╔═╡ 6b04fe4f-5078-4cf0-bad6-6ea01d4415b0
md"""
---
"""

# ╔═╡ 4618a663-f77e-4ccc-9b00-e84dbd5beb34
md"""
### Packages used in this notebook
"""

# ╔═╡ ec4b312d-c945-4d24-ad88-6fa15ed6f0f1
md"""
---
"""

# ╔═╡ 388f775e-c93a-4980-affe-5ba2521616a3
md"""
## Exercise 1. Expected inflation and the Phillips Curve
"""

# ╔═╡ a726c21b-578e-4167-94b0-4f364ac2be12
md"""
The Phillips curve, as presented by the Nobel prize winners Milton Friedman and Edmund Phelps, is written as: 

$$\pi=\pi^{e}-\omega\left(U-U_{n}\right) + \rho$$

where $\pi$ is the inflation rate, $\pi_{t}^{e}=\pi_{t-1}$ represents the expected inflation rate, $U$ is the unemployment rate, $U_n$ is the natural unemployment rate, and $\rho$ is some exogenous and temporary shock that may hit inflation (like rapid increases in oil prices, wars, natural disasters, pandemics, among others). The parameter $\omega$ gives the response of inflation to the unemployment gap: $U-U_n$.

👉 (a) What happens to inflation if private agents expect inflation to increase over time?
"""

# ╔═╡ fcea4d0b-a7c9-4d0b-b0da-1a903ed0bc8d
md"""
!!! answer "Answer (a)"

	If private agents expect inflation to increase over time $(\uparrow \pi^e)$, this will lead to higher inflation rates in the near future $(\uparrow\pi)$.

"""

# ╔═╡ 96090bb8-be09-48ac-8a8c-43c7f5f01808
md"""
👉 (b) In the following figure, we present evidence about inflation expectations in the US for 1978-2024. We use data from one of the most cited indexes of inflation expectations: "Surveys of Consumers, University of Michigan: Inflation Expectation© [MICH]". Considering the shaded areas, what do you expect to happen to inflation, given the information there?
"""

# ╔═╡ 0fed5c77-3cd8-4a32-a2d0-c5bd9cd4ba0e
md"""
!!! answer "Answer (b)"

	In the first shaded area (the early 1980s), private agents expected inflation to decrease $(\downarrow \pi^e)$, which should lead to a decline in actual inflation $(\pi)$. The second shaded area (from 2021 to the mid-2022) is the opposite.

"""

# ╔═╡ 5a8c4fea-e7e5-4713-808a-1e6ff167d6fa
md"""

👉 (c) What happens to expected inflation from the summer of mid-1983 to mid-1988? Did expected inflation cause any inflation pressures in this period?

"""

# ╔═╡ e1b4af55-b12d-4cfb-b2a6-9ef8e5311a70
md"""
!!! answer "Answer (c)"

	Expected inflation remained relatively stable over time. As expected inflation remained stable over this period, there were no pressures on inflation.

"""

# ╔═╡ 9bd58808-8f91-49f7-92a1-19bf05e7accc
md"""
---
"""

# ╔═╡ eb7f7dff-8f31-4793-bcdd-1e1c47363abb
md"""
## Exercise 2. Oil prices and inflation
"""

# ╔═╡ 8022fa16-02c3-4481-b7ba-2fc1ded4f344
md"""
In this exercise, we will discuss the impact of international oil prices on inflation. The Phillips curve, already discussed in Exercise 1, is written as: 

$$\pi=\pi^{e}-\omega\left(U-U_{n}\right) + \rho$$

where $\rho$ is intended to express some exogenous and temporary shock that may hit the economy and inflation, and one of those shocks in economists' minds is "oil prices".

👉 a) In the following figure, we plot the evolution of oil prices since the 1960s. Mention three aspects that may be relevant for the analysis of inflation in a market economy from the 1960s onwards.

"""

# ╔═╡ 27746311-ce1b-4b68-9bc2-8dcf1306a593
md"""
!!! answer "Answer (a)"

	1. Oil prices were very stable until the mid-1970s: oil prices should not affect inflation.

	2. From 1980 to around 1999, oil prices displayed a mild downward trend: inflation benefited from a slight decline in oil prices.

	3. From early 1999 onwards, oil prices have been tremendously unstable. Therefore, this instability should have brought significant instability to inflation.


	The entire world economy is largely dependent on fossil energy sources. Those tremendous swings in oil prices create terrible instability in production costs, leading to difficult-to-manage inflationary (or disinflationary) pressures in the economy.

"""

# ╔═╡ 84238089-0c10-4c03-a494-073e4d317673
md"""

👉 b) In the Phillips Curve, the Greek  letter $\rho$ is intended to reflect **_temporary_** shocks that hit inflation. The following figure shows the annual growth rates of oil prices. Do you see anything peculiar in that figure that may give support to the existence of such temporary shocks?

"""

# ╔═╡ 4dbc64ac-79c2-4308-87c4-5e57e3c1e7a2
md"""
!!! answer "Answer (b)"

	1. It is pretty frequent to see annual growth rates of oil prices greater than 50%. 

	2. Sometimes, the growth rate is even greater than 100%, like in April 2021, when it was 272%.

	3. Negative growth rates close to  --50% are also not uncommon.

	Therefore, we can confirm that those shocks are temporary: sometimes they go up, and sometimes they go down.

"""

# ╔═╡ 241b7098-b3f3-4e36-81d4-dafa822c49c9
md"""

👉 c) In the following figure, we show the relationship between inflation and the growth rate of oil prices for 2000-2019. What kind of relationship between those two variables do we have in that figure?

"""

# ╔═╡ 5ed4a452-13c4-49b1-8df8-f60b7ce3d905
#savefig(p17_10, "Oil_Inflation.svg", width=900, height=500);

# ╔═╡ 3ede19ba-d5c1-4313-bd70-2d0ba03fa02c
md"""
!!! answer "Answer (c)"

	1. It is clear that there is a positive relationship between the growth rate of oil prices and the inflation rate. 

	2. The causality is also clear: higher growth rates of oil prices will be translated into higher inflation rates.

	3. If oil prices do not go down, it seems complicated to control inflation just by increasing short-term interest rates.

"""

# ╔═╡ 31a42c8d-41db-4fdf-9af8-3740477294d1
md"""
---
"""

# ╔═╡ dcd13560-0f73-4aaa-b59a-b7961d8cdc63
md"""
## Exercise 3. The Unemployment Gap and increasing inflation
"""

# ╔═╡ 7f67d656-bb55-4097-ba49-10140f9b3502
md"""

This exercise will discuss the impact of the **_"Unemployment gap"_**, $(U-U_n)$, on inflation. Consider the following equations, which describe our well known Phillips Curve: 

$$\pi=\pi^{e}-\omega\left(U-U_{n}\right) \ , \quad \omega=1.2 \ , \quad \pi_{t}^{e}=\pi_{t-1}$$

 Assuming that, in this economy, the natural unemployment rate $U_n=4\%$, current unemployment is $U=4\%$, current inflation is $\pi=2\%$, and expected inflation is $\pi^e = 2\%$.

👉 (a) What happens to inflation over the next nine years if the central bank tries to bring the unemployment rate down to 3%, starting next year?

"""

# ╔═╡ beb52783-708e-4782-b271-643f2a4f3297
md"""
!!! answer "Answer (a)"

	As the central bank tries to reduce the unemployment rate from 4% to 3%, a value lower than the natural unemployment rate of 4% (the rate compatible with low and stable inflation), this will introduce an inflationary spiral: inflation increasing year after year.

	Let us see what happened in the first two years of the policy measure implementation:

	$$\pi_1 = 2 - 1.2 \times (3-4) = 3.2$$

	$$\pi_2 = 3.2 - 1.2 \times (3-4) = 4.4$$

	And so on. See the following figure for the entire set of values for the inflation rate.
"""

# ╔═╡ 66463532-449b-4cac-8083-8938786a193b
begin
	t = 10 # number of iterations we want to simulate
	ω = 1.2 # parameter
	Un = 4.0 # Natural rate of unemployment
	Uc = 3.0 # Unemployment rate chosen by the monetary authorities
	π1 = 2.0 # the initial condition we stipulate
	p = length(π1) # telling the routine how many initial conditions
	π = [π1  zeros(p,t-1)] # preallocation to increase performance

		 for i = 1 : t-1 # 9 iterations in the for loop
			 π[i+1] = π[i] - ω * (Uc - Un) 
		 end

	layout7_10 = Layout(;
					height = 450,
					title_text = "The Phillips Curve: inflation created by inflationary expectations",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
					hovermode="x",
        			#xaxis_range = [1,10],
        			xaxis_title = "Time",
					xaxis=attr(dtick=1, tick0=1),
        			#yaxis_range=[-2, 2], 
        			titlefont_size=18)
	plot(1:t, π', mode="markers+lines", marker_symbol="circle", marker_size="9",line_width=0.5, 
            		marker_color="Blue", layout7_10)
end

# ╔═╡ 466472a7-72f5-4ac2-9688-fc3fcdf3c09f
md"""
👉 (b) What causes this type of inflation dynamics?

"""

# ╔═╡ 28e51405-60fb-496c-b3cd-5aa256243b8e
md"""
!!! answer "Answer (b)"

	This type of inflation is caused by a "too hot" (or too strong) labor market that leads to inflationary expectations. The central bank allowed such expectations to take hold because it chose a rate of unemployment systematically below its natural level. From a different angle, the central bank has decided to keep the economy in a permanent economic boom.

"""

# ╔═╡ d1d2aaf7-d548-4b0d-9668-28ff262a8734
md"""
👉 (c) The monetary authorities have a target value for inflation: $\pi^{_T} =2 \%$. Is this target value compatible with 3% unemployment? What is the option available to the monetary authorities for achieving that target? 
"""

# ╔═╡ 57787a16-2568-4fcb-94cf-7f5533facf05
md"""
!!! answer "Answer (c)"

	Suppose the central bank decides at period 10 to reverse its initial decision and re-establish the unemployment rate equal to its natural level from the 10th period onwards. In that case, the economy for periods 11, 12, 13, ... will keep inflation at 12.8% (its value in period 10). 

	The only way to bring inflation back to its target value (2%) is to increase the unemployment rate to a level higher than its natural level. That is, the central bank should do precisely the opposite it did at the beginning; it created a boom, and now it has to force a recession. 

"""

# ╔═╡ a91c2b80-3215-46ac-938c-ef98e4dde437
md"""
---
"""

# ╔═╡ 95dce14a-b5f9-4dd8-af46-415e71b425fb
md"""
## Exercise 4. The Unemployment Gap and decreasing inflation
"""

# ╔═╡ 62d8a5a1-eab1-49ae-8947-b5299d1217c5
md"""

This exercise discusses the same problem as the previous one, with the difference that inflation will now decrease over time. Assume the same information as in the last exercise. 

👉 (a) What happens to inflation over the next nine years if the central bank tries to increase the unemployment rate to 5% starting next year?

"""

# ╔═╡ 6949d31b-c934-42f8-a53e-8fd1e068fff4
md"""
!!! answer "Answer (a)"

	As the central bank tries to increase the unemployment rate from 4% to 5%, a value higher than the natural unemployment rate of 4% (the rate compatible with low and stable inflation), this will introduce an deflationary spiral: inflation decreasing year after year.

	Let us see what happened in the first two years of the policy measure implementation:

	$$\pi_1 = 2 - 1.2 \times (5-4) = 0.8$$

	$$\pi_2 = 0.8 - 1.2 \times (5-4) = -0.4$$

	And so on. See the following figure for the entire set of values for the inflation rate.
"""

# ╔═╡ 95b7bd36-e0fc-4464-8bbe-9e9f491dae64
begin
	t_3 = 10 # number of iterations we want to simulate
	ω_3 = 1.2 # parameter
	Un_3 = 4.0 # Natural rate of unemployment
	Uc_3 = 5.0 # Unemployment rate chosen by the monetary authorities
	π1_3 = 2.0 # the initial condition we stipulate
	p_3 = length(π1_3) # telling the routine how many initial conditions are in the loop
	π_3 = [π1_3 zeros(p_3, t_3-1)] # preallocation to increase performance

		 for j = 1 : t_3-1 # the "for" loop
			 π_3[j+1] = π_3[j] - ω_3* (Uc_3 - Un_3) 
		 end

	layout7_11 = Layout(;height = 450,
					title_text = "Deflation created by deflationary expectations",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
					hovermode="x",
        			#xaxis_range = [1,10],
        			xaxis_title = "Time",
					xaxis=attr(dtick=1, tick0=1),
        			#yaxis_range=[-2, 2], 
        			titlefont_size=18)
	plot(1:t, π_3', mode="markers+lines", marker_symbol="circle", marker_size="9",line_width=0.5, 
            		marker_color="Darkblue", layout7_11)
end

# ╔═╡ 42f4b471-f8fe-4c75-a2dd-2383a4c683f2
md"""
👉 (b) What causes this type of inflation dynamics?

"""

# ╔═╡ 21d79f5f-f03a-4d63-8f79-d8868bd9234d
md"""
!!! answer "Answer (b)"

	This type of inflation is caused by a "too cool" (or too weak) labor market that leads to deflationary expectations. The central bank allowed such expectations to take hold because it chose a rate of unemployment systematically greater than its natural level. From a different angle, the central bank has decided to keep the economy in a permanent recession.

"""

# ╔═╡ bf93b0c6-ff58-4603-b0ac-57f802e473b1
md"""
👉 (c) The monetary authorities have a target value for inflation: $\pi^{_T} =2 \%$. Is this target value compatible with 5% unemployment? What is the option available to the monetary authorities for achieving that target? 
"""

# ╔═╡ 763bc186-dae2-479c-8195-6585b46733be
md"""
!!! answer "Answer (c)"

	The answer is precisely the opposite of the solution to question (c) in the previous exercise.

"""

# ╔═╡ 63ee9ae5-d84e-4b8d-bf65-88be736e3843
md"""
---
"""

# ╔═╡ f0196aef-6843-4f69-b387-ca5deec33604
md"""
## Exercise 5. The US labor market is too hot
"""

# ╔═╡ 10ccbe74-dff0-4f84-9b8d-3563559be3ba
md"""

According to the Pillips Curve concept, what does it mean in the following sentence, "the labor market is too hot for the Fed"?

>_“The labor market is too hot for the Fed [the unemployment rate falling to 3.5%]. I think it keeps them on a pretty aggressive tightening path. I expect another 75 basis point hike in November. They can hopefully moderate and reduce the size of rate hike to 50 basis points in December,” Kathy Bostjancic, chief U.S. economist at Oxford Economics, 7 October 2022._

"""

# ╔═╡ d333ae08-ecb5-454f-a963-2dc33583d21c
md"""
!!! answer

	This means that, for the Fed, in 2022, the unemployment rate was below the natural unemployment rate. The natural unemployment rate keeps the inflation rate low and stable. If the actual unemployment rate is lower than the natural rate, this will lead to inflation expectations and an upward trend in inflation.

	The only way the Fed can fight this inflationary pressure is to create a recession and increase the unemployment rate.

"""

# ╔═╡ d1cd84a3-10dd-4399-982b-52d149b9094c
md"""
---
"""

# ╔═╡ add07024-b7eb-445d-b4f4-6a6757c2ecd3
md"""
## Exercise 6. The Phillips Curve and real data
"""

# ╔═╡ bfb890be-fba4-4d55-9e39-a52d40a5adad
md"""

This exercise will focus on the Phillips curve's relationship between inflation and unemployment rates. We will use monthly data for those two variables, covering the US economy from 1960 to 2024.

👉 (a) In Fig 6.1, we plot the **_time series_** of the two variables: the inflation and unemployment rates. Can we infer if there is any kind of relationship between those two aggregates by looking at the time series?

"""

# ╔═╡ f1b510d9-e9b2-4db6-a738-ac8536f392f7
md"""
!!! note

	The shaded bars in Figure 6.1 represent the officially dated recessions in the US. The National Bureau of Economic Research (NBER) is the institution in the US that makes decisions about those dates, and they are usually taken as "official" dates. For the curious student, the NBER website is [here](https://www.nber.org/research/business-cycle-dating).
"""

# ╔═╡ 7478d0ce-4ae1-4949-9ae4-2ffa334454f9
md"""
!!! answer "Answer (a)"

	Time series is a plot that is not very good for spotting cross-relationships like we have in the Phillips curve. So, we have to look at a cross-plot, not a time series.

"""

# ╔═╡ 31b7a1b6-6849-41cc-92d0-f9c717bd3dda
md"""

👉 (b) In Fig 6.2 we plot the **_cross-plot_** between those two aggregates. Can you spot any relationship between those two variables that may resemble the "Phillips Curve"?

"""

# ╔═╡ 78475df7-5d81-410c-97bb-9ce06e897a7e
md"""
!!! answer "Answer (b)"

	A stable inverse relationship between inflation and unemployment -- what the Phillips Curve predicts if there are stable inflation expectations, a stable natural unemployment rate, and no supply shocks -- does not exist for the entire period under consideration. 

	It may exist for some periods, so let's see if that relationship subsists for subperiods. We will consider decades.


"""

# ╔═╡ 1aadb49f-b6f8-4b56-b6ba-c529d97ecd3f
md"""
👉 (c) Fig 6.3 presents the relationship for the 1960s. Do you see an apparent Phillips curve over there?

"""

# ╔═╡ 04b850b4-e659-4f1c-82be-ba912bad03af
md"""
!!! answer "Answer (c)"

	In the 1960s, the relationship expressed by the Phillips curve can be easily seen in this decade. The reason why this happened is very simple: oil prices remained extremely stable, and there were no relevant inflation expectations. So the curve was stable.

"""

# ╔═╡ 6ddb604e-3455-49d4-9298-ada4bf0227fc
md"""
👉 (d) In Figure 6.4, we get the 1970s relationship. What conclusions can we put forward?

"""

# ╔═╡ e9ac0a44-22ec-40d6-9fc1-2d9c8949455e
md"""
!!! note

	In the 1970s, we had two huge oil price shocks (1973 and 1979), a war in the Middle East in 1973, one oil embargo by Arabic countries in 1973-74 to Western economies, increasing inflation expectations, wages and price controls set by President Nixon in 1971 and abandoned in 1973, and a large recession in the mid-1970s.
"""

# ╔═╡ f8214ab3-f5f1-4d1d-9e2f-8a2fba5a929c
md"""
!!! answer "Answer (d)"

	It seems that all the major ingredients of the Phillips curve were forcing it to move in circles (loops). There was just too much happening to inflation and unemployment for a stable relationship in this curve to prevail in this decade.

	Increasing inflation expectations were shifting the curve to the right, and the same happened with the war in the Middle East, the oil embargo, and the two huge oil price increases. Wages and price controls were set to prevent the PC from moving to the right, but then they proved inefficient and were removed, leading to a jump in inflation again.

	On the other hand, after a while, the war ended, the embargo was over, oil price increases became mild, and there was a big recession in the mid-1970s, all these factors pressing the curve to shift to the left.

	The 1970s displayed all the complicated situations that may have led inflation to get out of control. What happened should be taken into good care to avoid something similar.

"""

# ╔═╡ c3fdfb74-1e85-487f-8715-0b083004aa03
md"""
👉 (e) In Figure 6.5, we get the 1980s relationship. What conclusions can we put forward?
"""

# ╔═╡ a17fc5ea-9598-40aa-be2f-864340b424a3
#savefig(p1_6, "PC_80s.svg", width=900, height=500)

# ╔═╡ 06a65cea-8587-4390-9f62-0adda6b8a84e
md"""
!!! note

	In the 1980s, four main facts may explain what happened to the Phillips curve. 
	
	First, President Jimmy Carter nominated Paul Volcker as the Fed chairman in 1979. Contrary to his predecessors, Volker disliked inflation a lot! In a conversation with President Jimmy Carter, Volker is quoted as saying:

	>_"If you appoint me, I will raise interest rates significantly and choke off the inflation, which will have bad political repercussions for you, Mr. President."_

	Volker's prophecy proved correct: He raised interest rates massively, causing a large recession, a large increase in unemployment, and a significant decrease in inflation. In late 1980, President Jimmy Carter lost re-election to Ronald Reagan.

	Second, oil prices declined on a sustained basis throughout the entire 1980 decade.

	Third, expected inflation declined abruptly in the early 1980s, becoming relatively stable over the rest of the decade.

	Fourth, the natural unemployment rate $(U_n)$ declined on a sustained basis after 1982 (see Exercise 8 below).

	"""

# ╔═╡ 24f78b7c-7bda-4ea1-b593-bdacb94e8c4d
md"""
!!! answer "Answer (e)"

	In the 1980s, the Phillips Curve (PC) displayed two different slopes: one between early 1980 and late 1982 (with a typical downward-sloping PC) and the other between late 1982 and 1989 (looking like a flat PC).

	One explanation has been widely accepted as a good one. Paul Volker (the Fed chairman) gave inflation in the US an ice bath by dramatically increasing interest rates, causing a significant recession and a big increase in unemployment. Volker's plan explains the negative slope of the PC above.

	The part that looks like a flat PC can be explained by three factors: inflation expectations became stable, oil prices fell systematically during the entire decade (see Exercise 2 above), and the Natural Unemployment rate $(U_n)$ declined from 1982 to 1989 (see Exercise 8 below). All these factors shifted the curve to the left, leading to the impression that the curve became flat.

"""

# ╔═╡ bb5398f5-516c-4954-8cc4-b0e1c1b6502c
md"""
---
"""

# ╔═╡ 86f54c54-58a5-4faf-9688-baddaa032ed2
md"""
## Exercise 7. The Okun's law
"""

# ╔═╡ c0bb7086-533d-4c95-850a-b445b877d8c8
md"""

*From the textbook.*


Suppose the Okun’s law can be expressed according to the following formula:

$$U - U_n = -0.5(Y - Y^P).$$ 

Assuming that potential output grows at a steady rate of 2.5% and that the natural rate of unemployment
remains unchanged:

👉 (a) Calculate by how much unemployment increases when real GDP decreases by one percentage point.

"""

# ╔═╡ f261756b-856c-429f-93df-af0980e535b2
md"""
!!! answer "Answer (a)"

	Rewritting the Okun's law in terms of discrete changes we get

	$$\Delta U- \Delta U_{n}=-0.5 \left(\Delta Y- \Delta Y^{P}\right).$$

	If $\Delta Y^{P}=2.5\%$ and $\Delta U_n =0\%$, and if $\Delta Y =-1\%$, then 

	$$\Delta U=-0.5(-1\%-2.5\%)=+1.75\%$$
"""

# ╔═╡ 9d8be89f-9ebe-45f0-b4b3-a74f7e3ee9de
md"""
👉 (b) Calculate by how much real GDP increases when unemployment decreases by two percentage points.
"""

# ╔═╡ a66fed79-59fe-4993-81c2-acbff747332a
md"""
!!! answer "Answer (b)"

	Rewritting the Okun's law in terms of discrete changes, and if $\ \Delta Y^{P}=2.5\%$,  $\Delta U_n =0\%$, and $\Delta U =-2\%$, then 


	$$-2\% - 0\% = -0.5 (\Delta Y- 2.5\%)$$


	Then

	$$\Delta Y=3.25\%/0.5=+6.5\%$$
"""

# ╔═╡ 5fa00216-1464-4f2d-80cf-4d333d9dced3
md"""
---
"""

# ╔═╡ 3cf86819-09f0-43d3-aff7-494be276bf0f
md"""
## Exercise 8. Cyclical unemployment & the output-gap
"""

# ╔═╡ ccc8aa66-cb8d-4bbe-8457-f496f7bff47d
md"""

This exercise will use quarterly data to examine Okuns' law in the USA between 1960 and 2019.

👉 (a) In Fig 8.1 and 8.2, we plot the evolution of the unemployment rate, its trend, and the cyclical unemployment rate. What is cyclical unemployment? How can we obtain it?
"""

# ╔═╡ 208bbede-ad8d-45f0-8524-43948733ffb8
#savefig(p7_14, "Cyc_Unemp.svg", width=900, height=500)

# ╔═╡ dc5bfa8c-7247-44fc-8837-a7fa32c36d54
md"""
!!! answer "Answer (a)"

	Cyclical unemployment is the type of unemployment that is caused by short-term business cycles (recessions and booms). If the economy is in a boom, actual unemployment will be below its long-term trend (the blue line below the red one), while it will be above its long-term trend if the economy is in a recession. 

	It is the only type of unemployment that can be reduced (or increased) by the intervention of policy institutions (government and central bank), when they try to manage the business cycle.

	It is obtained by the difference between the actual unemployment rate and its long-term trend.

"""

# ╔═╡ d040f3bb-7ea9-4000-813a-6e1f4e0163cb
md"""

👉 (b) In Figures 8.3 and 8.4, we plot the evolution of the real GDP, its trend, and the output gap. What is the output gap? How can we obtain it? 

"""

# ╔═╡ ea2e3d0e-c933-4d67-ac24-3ba9bcf6fc8a
md"""
!!! answer "Answer (b)"

	The output gap is the percentage deviation of real GDP from its long-term trend. Its values tell us whether an economy is in a recession (highly negative values), or a boom (highly positive values)

	It is obtained by the difference between real GDP and its long-term trend.

"""

# ╔═╡ 2ab10860-0671-4e3c-a7ea-dff6ae80c528
md"""

👉 (c) Fig 8.5 displays Okun's law for the US. For the period under consideration (1960.Q1-2019.Q4), the Okuns's law has an intercept equal to $0$ and a slope equal to $-0.441393$ (see figure below). What is this law? How do we obtain it?

"""

# ╔═╡ cc770177-1d7a-4f6e-a930-c805b3bfee6d
md"""
!!! answer "Answer (c)"

	Okun's law is an empirical fact: if the economy loses 1% of the output gap, cyclical unemployment tends to increase by (around) 0.5%. In this particular period, the parameter is not -0.5 but -0.44. 

	It is obtained by passing (or regressing) a straight line across the cloud of points. Those points on the left side and those on the right side should be balanced.

"""

# ╔═╡ 4d7901da-39a9-4f24-830f-0ecd945d41dd
md"""
---
"""

# ╔═╡ 03706c92-80c5-41d7-bbc3-4e68d1873e77
md"""
## Exercise 9. The short-run AS curve
"""

# ╔═╡ 709997bf-ae42-49e8-8224-b9c2aae30bdb
md"""

*From the textbook.*

Assuming that the Okun’s law is given by $U - U_n = -0.5 (Y - Y^P)$ and that the Phillips curve is given by
$\pi=\pi^{e}-0.6(U-U_n)+\rho$

👉 (a) Obtain the short-run aggregate supply curve if expectations are adaptive, inflation was 3% last year, and potential output is \$10 trillion (assume $\rho=0$).

"""

# ╔═╡ df7eac78-5c6c-4307-b2ae-ad2331d99c4a
md"""
!!! answer "Answer (a)"

	The short-run Phillips curve can be easily obtained (remember we work with percentage points: $3\%$ is 3 pp):

	$$\pi= 3 + 0.6 \times 0.5 (Y - 10)+0$$

"""

# ╔═╡ b0c297b8-b49c-4b45-b294-1f865d6ca2d2
md"""
👉 (b) What is the inflation rate when GDP is 8, 10, and 12 trillion dollars? (Note: we can answer by looking at the figure below or the numerical output provided by the code cell below).

"""

# ╔═╡ 7ee61691-f0ae-40c9-8d42-2ca45a3317d2
begin

	inflation(gdp) = 3.0 .+ (0.6 .* 0.5) .* (gdp .- 10.0) .+ 0.0 	#this function represents the AS curve
	gdp = [8 , 10 , 12] 										 	# vector containig the values of GDP
	inflation(gdp)													# prints the values of inflation
		
end

# ╔═╡ 926e7520-8c1e-4dc2-a9a5-0bafcd13bf47
begin
	output = 0:1:16
	
	PC = 3.0 .+(0.6 .* 0.5).*(output .- 10.0)
	
	trace7_24 = scatter(;x =output, y = PC, 
					name = "AS", line_color = "Blue", mode = "lines", 
					line_width="2.0")
		
	trace7_25 = scatter(; x = [8.0, 10.0, 12.0], y = [2.4, 3.0 , 3.6], text = ["1", "2" , "3"], 
					textposition = "top center", name = "", mode="markers+text", marker_size= "12",
					marker_color="blue",	
					textfont = attr(family="sans serif", size=16, color="black"))
	
	layout7_25 = Layout(; height = 450,
					title_text = "The AS curve",
					title_x = 0.5,
        			yaxis_title = "Rate of Inflation",
					yaxis_zeroline= false,
					hovermode="x",
        			xaxis_range = [4.0, 16],
        			xaxis_title = "GDP (trillion dollars)",
					xaxis_zeroline= false,
        			yaxis_range=[1, 5], 
        			titlefont_size=18)

	
	p7_25=plot([trace7_24, trace7_25], layout7_25)
end

# ╔═╡ 66b70b7b-c792-44b7-8045-68ad836d1616
md"""
!!! answer "Answer (b)"

	When GDP is equal to 8, 10, and 12 trillion dollars, inflation will be equal to 2.4%, 3%, and 3.6%.

	We can calculate these values with a simple calculator or use the notebook directly. To use the notebook, we can write the AS function, give values for GDP, and print the values of inflation. See the cell below.

"""

# ╔═╡ 1b35a651-08b3-4598-b830-b5bc9d73df34
md"""
👉 (c) The figure above plots the short-run AS curve with three points. What happens to the unemployment rate when we move from points 1 to 2 and 3?
"""

# ╔═╡ 506e4667-627c-4808-83be-ef9fe2211f87
md"""
!!! answer "Answer (c)"

	Moving from points 1 to 2 and 3 means that the inflation rate is increasing. If inflation is increasing, the unemployment rate should be declining by the Phillips Curve.

	Obviously, this conclusion is correct if we assume that everything else remains constant. This "everything else constant" means that: (i) there are no supply shocks, (ii) no changes in inflation expectations, (iii) potential real GDP remains constant.

"""

# ╔═╡ c3fc8216-c016-4346-b7bd-1c4b421f2b37
md"""
---
"""

# ╔═╡ 051bf8f6-d581-4a51-955f-02d78446c0f1
md"""
## Exercise 10. A comprehensive exercise
"""

# ╔═╡ df9417d2-ebe1-498c-b299-f2b8fdee1084
md"""

Consider the AS function as described in the slides and textbook:

$$\pi=\pi^{e}+\gamma\left(Y-Y^{P}\right)+\rho.$$

The following information is known:

$$\gamma=4.5 \quad , \quad Y^{P}=14.0 \quad , \quad  \pi^{e}=2.0 \quad , \quad \rho=0.0$$

Firstly, we should pass the parameters and exogenous variables to the notebook (index 8 is used to tell that these are the values of exercise 8). Secondly, answer the following questions:

"""

# ╔═╡ b28d588a-b4ff-444b-8b16-3cd05110ad1d
γ9 = 4.5 ; Yᴾ9 = 14.0 ; πᵉ9 = 2.0 ;	ρ9 = 0.0; 		#paremeters and exogenous variables 

# ╔═╡ 7087371c-98d3-4e8b-a0e8-47c0631ee59e
md"""
👉 a) In the following figure, we plot the short-run AS curve for GDP values between 12.5 and 15.5 trillion dollars. We also plot the long-run AS curve. What do they represent?
"""

# ╔═╡ 90c6188b-4356-415a-94ce-dc978d3b15ac
begin
	
	Y9 = 12.0:0.01:16.0
	
	π9 = πᵉ9 .+ γ9 .*(Y9 .- Yᴾ9) .+ρ9
	
	trace7_40 = scatter(; x = Y9, y = π9, mode="lines" , line_color = "Red", line_width = "3",
					name  = "AS")
	
	trace7_41 = scatter(; x = [13.5, 14.0, 14.5], y = [-0.25, 2.0, 4.25], text = ["1", "2" , "3"], 
					textposition = "top center", name = "", mode="markers+text", marker_size= "12",
					marker_color="Red",	
					textfont = attr(family="sans serif", size=16, color="black"))
	
	trace7_49 = scatter(; x = [14,14] , y = [-7,14] , mode = "line", line_width = "4", name = "Yp", 
					line_color = "Orange")
	
	layout7_41 = Layout(; height = 450,
					title_text = "The AS function", titlefont_size=18,
					title_x = 0.5,
                     xaxis=attr(title=" GDP trillion dollars (Y)", showgrid=true, zeroline=false),
					 hovermode="x",
                     xaxis_range = [12.5, 15.5],
                     yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					 yaxis_range = [-5.0, 10.0])
	
	p7_41 = plot([trace7_49, trace7_40, trace7_41], layout7_41)
	
end

# ╔═╡ 03c7091d-7830-4fad-b444-413454736d5b
md"""
!!! answer "Answer (a)"

	The short-term AS curve (AS) gives the level of real GDP for each level of inflation, assuming that inflation expectations $(\pi^e)$, potential real GDP $(Y^P)$, and shocks $(\rho)$ all remain constant. It tells us that there is a trade-off between inflation and real GDP: on the supply side, more GDP can only be achieved by accepting higher inflation.

	The long-term AS curve (Yp) gives the level of potential real GDP, which is a vertical curve. It says that in the long-term, there is no trade-off between inflation and real GDP: only technology, labor, or capital may change the level of potential tral GDP.

"""

# ╔═╡ 0735c5c2-f79c-4424-8fac-e060280d904c
md"""

👉 b) What makes the economy move from points 1 to 2 and 2 to 3?

"""

# ╔═╡ 239ccedb-f91d-4eb1-9a69-588bf4ff4ebc
md"""
!!! answer "Answer (b)"

	1. From points 1 to 2: inflation in the economy increases by 2 percentage points (from 0% to 2%).

	2. From points 2 to 3: inflation in the economy increases by 2 percentage points (from 2% to 4%).

	3. These movements represent a **movement along** the AS curve.

"""

# ╔═╡ 8fc49bd4-075d-4e3f-b50b-5f381c09ff10
md"""
👉 c) Suppose there is a shock in oil prices such that $\rho=3.0$. Using the slider **ρ8c** below, represent the impact of this shock upon the initial functions.

"""

# ╔═╡ 8430162f-4f13-4525-9260-027d84222808
md"""
`ρ9c = `$(@bind ρ9c Slider(-6.0:0.25:6.0, default=0.0, show_value=true))
"""

# ╔═╡ ae1c8887-678f-4485-a492-a1b06e7aaefa
begin
	
	#Y8 = 12.0:0.01:16.0
	
	π9Δρ = πᵉ9 .+ γ9 .*(Y9 .- Yᴾ9) .+ ρ9c
	
	trace7_43 = scatter(; x = Y9, y = π9Δρ, mode="lines" , line_color = "Gray", line_width = "3",
			name  = "AS2")
	
	trace7_44 = scatter(; x = [13.5, 14.0, 14.5], y = [2.75, 5.0, 7.25], text =["1'", "2'" , "3'"], 
					textposition = "top center", name ="", mode="markers+text", marker_size= "12",
					marker_color="Gray", 
					textfont = attr(family="sans serif", size=16, color="black"))
	
	trace7_45 = scatter(; x = [12.83333, 13.33333, 13.833333], y = [-0.25, 2.0, 4.25], 
					text =["1''", "2''" , "3''"], 
					textposition = "top center", name ="", mode="markers+text", marker_size= "12",
					marker_color="Orange", 
					textfont = attr(family="sans serif", size=16, color="black"))
	
	layout7_44 = Layout(; height = 450,
					title_text = "The AS function with a negative supply shock", titlefont_size=18,
					title_x = 0.5,
                     xaxis=attr(title=" GDP (trillion dollars)", showgrid=true, zeroline=false),
                     xaxis_range = [12.5, 15.5],
					 hovermode="x",
                     yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					 yaxis_range = [-5.0, 13.0])
	
	p7_44 = plot([trace7_49, trace7_43, trace7_40, trace7_44, trace7_41], layout7_44)
	
end

# ╔═╡ db2990be-db70-456f-96c4-9de707bfcfbd
md"""
!!! answer "Answer (c)"

	There is no information on whether the shock is temporary or permanent. However, since we are talking about oil prices, it makes sense to think that the shock is temporary. 

	In the previous exercise, we saw that oil prices are incredibly volatile, with positive and negative growth rates, even from month to month. So, if the shock is temporary, only the short-run AS function will be affected.

	The increase in oil prices, increasing ρ8 from 0% to 3%, shifts the short-run AS curve to the left, from points 1, 2, 3 to 1', 2', 3'. For every level of real GDP, the economy will display higher inflation.

"""

# ╔═╡ 4c73146a-3e09-4125-8cfd-5665206d68b6
md"""

👉 d) Is this oil price shock a positive or negative shock?
"""

# ╔═╡ 7d77dff4-dbe2-4ee0-9be3-fbdaee9edaa8
md"""
!!! answer "Answer (d)"

	The textbook treats this type of shock as **negative** because it reduces GDP for every level of inflation. From this perspective, a negative shock will move the economy from points 1, 2, 3 into points 1'', 2'' and 3'', in the following figure. To avoid confusion, this will also be the approach we will follow.

	Be careful with the words chosen to frame a particular exercise: a "negative" supply shock means inflation will go up; a "positive" supply shock means inflation will go down.


"""

# ╔═╡ f3b6fae7-44de-496d-98a3-11f416e62f20
md"""
`ρ9d = `$(@bind ρ9d Slider(-6.0:0.25:6.0, default=0.0, show_value=true))
"""

# ╔═╡ 34161592-b86c-428e-ad0f-6c7b6c8c1be5
begin
	
	#Y8 = 12.0:0.01:16.0
	
	π9Δρ_d = πᵉ9 .+ γ9 .*(Y9 .- Yᴾ9) .+ ρ9d
	
	trace7_53 = scatter(; x = Y9, y = π9Δρ_d, mode="lines" , line_color = "Gray", line_width = "3",
			name  = "AS2")
	
	
	trace7_55 = scatter(; x = [12.83333, 13.33333, 13.833333], y = [-0.25, 2.0, 4.25], 
					text =["1''", "2''" , "3''"], 
					textposition = "top center", name ="", mode="markers+text", marker_size= "12",
					marker_color="Orange", 
					textfont = attr(family="sans serif", size=16, color="black"))
	
	layout7_54 = Layout( height = 450,
					title_text = "The AS function with a negative supply shock", titlefont_size=18,
					title_x = 0.5,
                     xaxis=attr(title=" GDP (trillion dollars)", showgrid=true, zeroline=false),
                     xaxis_range = [12.5, 15.5],
					 hovermode="x",
                     yaxis=attr(title="Rate of inflation (π)", zeroline = false),
					 yaxis_range = [-5.0, 13.0])
	
	p7_54 = plot([trace7_49, trace7_53, trace7_40, trace7_55, trace7_41], layout7_54)
	
end

# ╔═╡ aafdc1f7-7f2a-454c-b3da-9c9c68aed107
md"""
---
"""

# ╔═╡ b210c2e6-c9a5-44d8-a01c-c9702da0606d
md"""
## Exercise 11. Was the US labor market "too hot" in 2022? (optional)
"""

# ╔═╡ aea3a28e-c408-4a6d-ac6a-1507ed4e4d1b
md"""
>*"Most seers believe the US labor market is overly strong. Federal Reserve Chair Jerome Powell recently described it as “extremely tight” and “out of balance,” with demand for workers far outstripping supply.  I beg to differ."* Gary Shilling, Bloomberg, 4 October 2022


As usual, in macroeconomics, there is a massive controversy about the current situation of the US labor market. If the US labor market were "so hot", how can it be that real wages have been declining since 2021? Or how can the participation rate still be below the pre-pandemic level? We do not have much space here, but just a hint in the following figure: in February 2020 (before the pandemic started), unemployment as 3.5% and inflation was 2.46%; in September 2022, unemployment was 3.5% (!!), while inflation reached 8.22%. How is this compatible with a too-hot labor market? Gary Shilling does not see how the two facts can be conjugated to deliver such a message: 

"""

# ╔═╡ c1a3d11b-881c-4c74-ac1d-e794562349e0
md"""
>*"My analysis indicates that current labor market stringency isn’t due to robust demand for workers but because Americans are limiting the supply after the pandemic. After staying at home for two years due to the lockdowns, many people became accustomed to working remotely and putting in fewer hours while others dropped out, retiring early. In late September, office occupancy as measured by workers at actual desks was only 47.5% of early 2020, according to Kastle Systems."* [here](https://www.bloomberg.com/opinion/articles/2022-10-04/the-us-labor-market-is-looser-than-it-appears)
"""

# ╔═╡ 854c4538-8d47-41f6-ba61-edfd81f632d1
md"""
!!! answer "Suggested answer"

	The official position of the Fed is that inflation is too high, and the "too hot" labor market contributes a lot to that situation. So, the Fed has decided to increase short-term interest rates until inflation comes back to 2% (this is the Fed's desired inflation level).

	What is the problem with the Fed's official position? Suppose inflation is not the result of a "too hot" labor market but instead of something else, like oil prices, Covid19 disruptions of supply chains, and a war in Ukraine. If this is the case, the Fed may increase interest rates in succession, creating a recession, but inflation will stay very high. The Fed is fighting a perceived disease, while the actual problem of the US economy is different.

	And there is plenty of evidence that the labor market is not the factor that is pushing inflation up:

	- When inflation started to increase on a systematic basis (May 2020), unemployment was at 13.2%;

	- Unemployment is now at 3.5%, the same rate as before the Covid19 stroke, but inflation is 3.5 higher than it was then.

	- There is irrefutable evidence that inflation increased when oil prices started to grow at astonishingly high rates in the summer of 2020.

	So, the Fed may be applying the wrong remedy to the economy. The Fed is convinced that it is doing the right thing, many disagree, and they may have a point.

"""

# ╔═╡ 5262ff53-633c-46c9-b812-f31d8c90a873
md"""
---
"""

# ╔═╡ 9cd31e46-2efd-4cb7-a1fb-833fb0214d84
md"""
## Auxiliary cells (do not delete)
"""

# ╔═╡ 9fa87191-7c90-4738-a45a-acd929c8bd1b
  TableOfContents()

# ╔═╡ b9c0b1d6-2289-472d-92ce-576c79f73268
md"""
##### Data files used in this notebook

- NBER_Recessions.csv
- Mich.csv
- ASdata2.csv
- US_data_May2021.csv

These CSV files must be kept in the same folder as this notebook.

"""

# ╔═╡ f208e537-179c-444f-ba47-96b15ce74855
md"### CSS code"

# ╔═╡ 9dcec3b2-c378-4814-afed-6997be743699
html"""
<style>
	@media screen {
		main {
			margin: 0 auto;
			max-width: 1550px;
    		padding-left: max(100px, 10%);
    		padding-right: max(380px, 10%); 
            # 383px to accomodate TableOfContents(aside=true)
		}
	}
</style>
"""

# ╔═╡ d68f00b3-1e31-44bb-b48c-d53580c8893d
md"""
### Exercises
"""

# ╔═╡ d9aaf9ef-c3aa-4c9f-9762-e1d818ed0edd
md"""
#### Exercise 1. Supporting cells (Expected inflation and ...)
"""

# ╔═╡ 7ab54cda-93a0-4969-870d-4dc15879a8a7
# Define a function to generate recession shading traces
    function recession_trace(dates, recession_data; fillcolor="gray") #fillcolor="rgba(255, 0, 0, 0.8)"
        # Function to find start and end dates of recession periods
        function find_recession_periods(dates, recession_data)
            periods = []
            start_idx = 0

            for i in 1:length(recession_data)
                if recession_data[i] == 1 && start_idx == 0
                    start_idx = i  # Start of recession
                elseif recession_data[i] == 0 && start_idx != 0
                    push!(periods, (dates[start_idx], dates[i-1]))  # End of recession
                    start_idx = 0
                end
            end

            # Handle if the series ends in a recession
            if start_idx != 0
                push!(periods, (dates[start_idx], dates[end]))
            end

            return periods
        end

        # Get recession periods from the data
        recession_periods = find_recession_periods(dates, recession_data)

        # Create scatter traces for shaded recession areas
        traces = []
        for period in recession_periods
            start = period[1]
            stop = period[2]
            trace = scatter(; x=[start, stop, stop, start], 
                            y=[-4, -4, 100, 100], 
                            fill="toself", 
                            fillcolor=fillcolor, 
                            opacity=0.3, 
                            line=attr(width=0), 
                            showlegend=false, 
                            name="Recession", 
                            hoverinfo="skip",  # Skips hover info for clean visuals
                            xaxis="x", 
                            yaxis="y")
            push!(traces, trace)
        end

        return traces
    end;

# ╔═╡ af29458d-2ee7-406b-bbc9-8b9a2e94fa39
Mich = CSV.read("Mich.csv", DataFrame);

# ╔═╡ a3a7ca81-c5ad-4ee5-9cc9-65e017c45bb3
begin
	fig10=plot(Mich.Months, Mich.MICH, line_color = "blueviolet")# Mich.EXPINF1YR])
	relayout!(fig10, height = 450, 
		title_text = "Inflation expectations in the USA (Michigan Survey)", 
		title_x = 0.5, yaxis_title = "Percentage points",
		hovermode="x unified",)
 	add_vrect!(fig10, "1980-01-01", "1983-03-01", fillcolor="red", opacity=0.3, line_width=0; row=1, col=1)
    add_vrect!(fig10, "2020-12-01", "2022-06-01", fillcolor="red", opacity=0.25, line_width=0; row=1, col=1)
	fig10
end

# ╔═╡ 8205aa02-e1a1-433a-a4ff-7491dd07faa7
md"""
#### Exercise 6. Supporting cells (The Phillips curve and ...)
"""

# ╔═╡ 51afa448-613b-4a9f-8af8-a0b02abba89c
recess = CSV.read("NBER_Recessions.csv", DataFrame);

# ╔═╡ 518caed6-cb92-4410-bc0a-7a5c7bdc155e
ASdata = CSV.read("ASdata2.csv", DataFrame);

# ╔═╡ 5ea54f91-bd86-4454-b6ef-3a97a0105ec2
begin
	trace7_61 = scatter(;x = ASdata.Months, y = ASdata[:,4], name = "Oil", line_color = "Blue", 
						mode = "markers+lines", line_width="0.3", marker_size ="4")

	layout7_61 = Layout(; height = 450,
		
				title_text = "Growth rate of oil Prices in International Markets (1960--2024)",
				title_x = 0.5,
				xaxis_range = ["1959-01-01" , "2025-06-01"],
		
				hovermode="x unified",
		
            	xaxis = attr(
                title = "Monthly obervations",
                #tickformat = "%Y",
                #hoverformat = "%Y-M%m",
                #tick0 = "1970/01/01",
                #dtick = "M120",
        				),
        		#xaxis_range = [1960, 2020],
        		yaxis_title = "Growth rate",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	p7_61 = plot([trace7_61], layout7_61)
end

# ╔═╡ 8440209f-17c5-4ed7-9a98-8a5a2905a694
let

    # Example data (Replace with actual data)
    start_date = Date("1960-01-01")
    end_date = Date("2024-08-01")
    dates = collect(start_date:Month(1):end_date)
    recession_data = recess.Recessions[145:end]  # Replace with actual recession data

    # Get the recession traces
    recession_traces = recession_trace(dates, recession_data)

    # Example plot with another time series (e.g., unemployment)
    unemployment_data = ASdata.UR  # Replace with actual data
    trace_unemployment = scatter(; x=dates, y=unemployment_data, mode="lines", line_color="red", line_width = "1.8", name="UR")

	inflation_data = ASdata.CPI  # Replace with actual data
    trace_inflation = scatter(; x=dates, y=inflation_data, mode="lines", line_color="blue", line_width = "1.8", name="CPI")

    # Combine the recession traces with other data
    figx = plot([[trace_unemployment, trace_inflation]; recession_traces...])

    # Adjust layout
    relayout!(figx,
		height = 450,
        title_text="Fig 6.1. Inflation Rate (CPI) versus Unemployment Rate (UR): USA (1960-2024)",
        title_x=0.5,
        hovermode="x unified",
		xaxis_range = ["1959-06-01", "2024-12-01"],
		yaxis_range = [-2.4, 16],
        xaxis=attr(title="Date", showgrid=true, zeroline=false),
        yaxis=attr(title="Percentage", zeroline=false),

		annotations=[ attr(x="1968-10-01", y=4.5, text="Shaded bars: NBER recessions", showarrow=false, yshift=190, 
			font=attr(family="Calibri",	size=15, color="black" ))
	
			]
    )

    # Display the figure
    figx
end

# ╔═╡ 778b7504-2f80-480e-bf43-cdbea4be4514
begin
	trace10_1 = scatter(;x = ASdata.Months[601:end], y = ASdata[601:end,3], name = "UR", line_color = "Blue", 
						mode = "lines", line_width="2", marker_size ="0")
	
	trace10_2 = scatter(;x = ASdata.Months[601:end], y = ASdata[601:end,2], name = "CPI", line_color = "Red", 
						mode = "lines", line_width="2", marker_size ="0")

	layout10_2 = Layout(;
				height = 450,
				title_text = "Inflation Rate (CPI) versus Unemployment Rate (UR): USA (2010--2024)",
				title_x = 0.5,
				hovermode="x",
		
            	xaxis = attr(
                	title = "Monthly obervations"
                	#tickformat = "%Y",
                	#hoverformat = "%Y-Q%q",
                	#tick0 = "1960/01/01",
               	 	#dtick = "M120"
				),
		
        		#xaxis_range = [1960, 2020],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	p0_2 = plot([trace10_1 , trace10_2], layout10_2)
end

# ╔═╡ f613e456-403b-4bff-bd65-315b1be5eb45
period1_1 = MonthlyDate(1960, 1):Month(1):MonthlyDate(2024, 9);

# ╔═╡ 0d2d4c66-b49c-46c0-8ed0-3906db9c8deb
begin
	trace7_51 = scatter(;x = Date.(period1_1), y = ASdata[:,5], name = "Oil", line_color = "Blue", 
						mode = "markers+lines", line_width="0.3", marker_size ="4")

	layout7_51 = Layout(; height = 450,
		
				title_text = "OIl Prices in International Markets (1960--2024)",
				title_x = 0.5,
				xaxis_range = ["1959-01-01" , "2025-06-01"],
		
				hovermode="x unified",
		
            	xaxis = attr(
                title = "Monthly obervations",
                #tickformat = "%Y",
                #hoverformat = "%Y-M%m",
                #tick0 = "1970/01/01",
                #dtick = "M120",
        				),
        		#xaxis_range = [1960, 2020],
        		yaxis_title = "Dollars per Barrel",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	p7_51 = plot([trace7_51], layout7_51)
end

# ╔═╡ fdec5907-d925-4ae2-9298-fd14f3213139
begin
	
	layout1_3 = Layout(; height = 550, width = 550,
					title_text = "Fig 6.2. Inflation versus Unemployment: USA (1960-2024)",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
        			#xaxis_range = [1960, 2020],
        			xaxis_title = "Rate of unemployment",
        			#yaxis_range=[-2, 2], 
        			titlefont_size=16)

	p1_3 = plot(ASdata[:,3], ASdata[:,2], mode="markers+lines", text=period1_1,
            	marker_symbol="circle", marker_size="6",line_width=0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color="BlueViolet",layout1_3)
	
end

# ╔═╡ f5ae46d3-3f3d-4231-ad25-4333af5949e3
period1_2 = MonthlyDate(1960, 1):Month(1):MonthlyDate(1969, 12);

# ╔═╡ 7bc32249-45ff-42bf-8d39-ec1a7670fbdd
begin
	
	layout1_4 = Layout(; height = 550, width = 550,
					title_text = "Fig 6.3. Inflation versus Unemployment: USA 1960s",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
        			#xaxis_range = [1960, 2020],
        			xaxis_title = "Unemployment rate",
        			#yaxis_range=[-2, 2], 
        			titlefont_size=16)

	p1_4 = plot(ASdata[1:120,3], ASdata[1:120,2], mode="markers+lines", text=period1_2,
            	marker_symbol="circle", marker_size="6.5",line_width=0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color="Blue",layout1_4)
end

# ╔═╡ 2625f138-a5e6-42a3-add7-c648b7334b57
period1_3 = MonthlyDate(1970, 1):Month(1):MonthlyDate(1979, 12);

# ╔═╡ 24f2f06e-05fe-4b12-b238-d660f449ee9b
begin
	
	layout1_5 = Layout(; height = 550, width = 550,
					title_text = "Fig 6.4. Inflation versus Unemployment: USA 1970s",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
        			#xaxis_range = [1960, 2020],
        			xaxis_title = "Unemployment rate",
        			#yaxis_range=[-2, 2], 
        			titlefont_size=16)

	p1_5 = plot(ASdata[121:240, 3], ASdata[121:240, 2], mode="markers+lines", text=period1_3,
            	marker_symbol="circle", marker_size="6.5",line_width=0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color="Darkblue",layout1_5)
end

# ╔═╡ 5e8b862d-031c-48e4-8abd-8532637d04a2
period1_4 = MonthlyDate(1980, 1):Month(1):MonthlyDate(1989, 12);

# ╔═╡ 793b3108-76c1-4274-9861-c9ee948eca88
begin
	layout1_6 = Layout(; height = 550, width = 550,
					title_text = "Fig 6.5. Inflation versus Unemployment: USA 1980s",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
        			#xaxis_range = [1960, 2020],
        			xaxis_title = "Unemployment rate",
        			#yaxis_range=[-2, 2], 
        			titlefont_size=16)

	p1_6 = plot(ASdata[241:360, 3], ASdata[241:360, 2], mode="markers+lines", text=period1_4,
            	marker_symbol="circle", marker_size="6.5",line_width=0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color="Darkred",layout1_6)
end

# ╔═╡ cb1e02e1-9baf-4e47-b5bf-ecf472ad6fde
period1_6 = MonthlyDate(2000, 1):Month(1):MonthlyDate(2009, 12);

# ╔═╡ 1350111b-499c-4079-aa9d-103c081e0e74
period1_7 = MonthlyDate(2010, 1):Month(1):MonthlyDate(2019, 12);

# ╔═╡ ba40d0ff-992c-44de-b1a2-7ffe030bb3fd
period17_10 = MonthlyDate(2000, 1):Month(1):MonthlyDate(2022, 9);

# ╔═╡ 789b9f2b-4d0c-40b9-9ca4-809abbdfcba2
begin
	layout17_10 = Layout(; height = 550, width = 550,
					title_text = "Inflation vs Oil prices growth rate (USA: 2000-2019)",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
        			xaxis_range = [-80, 150],
        			xaxis_title = "Oil prices growth rate",
        			#yaxis_range=[-80, 2], 
        			titlefont_size=16)

	p17_10 = plot(ASdata[361+120:720,4], ASdata[361+120:720,2], mode="markers", text=period17_10,
            	marker_symbol="circle", marker_size="7",line_width=0.5, 
            	marker_color="brown", 
				layout17_10)
end

# ╔═╡ aa86d58a-214d-4ef3-a2a3-86b78af55967
# ╠═╡ disabled = true
#=╠═╡
begin
	layout1_7 = Layout(; height = 550, width = 550,
					title_text = "Fig 06. Inflation versus Unemployment: USA 2000s",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
        			#xaxis_range = [1960, 2020],
        			xaxis_title = "Unemployment rate",
        			#yaxis_range=[-2, 2], 
        			titlefont_size=16)

	p1_7 = plot(ASdata[481:600, 3], ASdata[481:600, 2], mode="markers+lines", text=period1_6,
            	marker_symbol="circle", marker_size="6,5",line_width=0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color="Purple",layout1_7)
end;
  ╠═╡ =#

# ╔═╡ af88f312-87dd-4a1e-913d-31ba12f06b45
# ╠═╡ disabled = true
#=╠═╡
begin
	layout1_9 = Layout(; height = 550, width = 550,
					title_text = "Fig 07. Inflation versus Unemployment: USA 2010-2019",
					title_x = 0.5,
        			yaxis_title = "Rate of inflation",
        			#xaxis_range = [1960, 2020],
        			xaxis_title = "Unemployment rate",
        			#yaxis_range=[-2, 2], 
        			titlefont_size=16)

	p1_9 = plot(ASdata[601:720, 3], ASdata[601:720, 2], mode="markers+lines", text=period1_7,
            	marker_symbol="circle", marker_size="6.5",line_width=0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color="olive", layout1_9)
end;
  ╠═╡ =#

# ╔═╡ f4d5a7be-361c-4a52-b501-b363548e42ee
# ╠═╡ disabled = true
#=╠═╡
md"""
!!! answer "Suggested answer (e)"

	It is not easy to understand what has happened during the 2000s and since 2010 to the Phillips Curve (PC). Probably, nobody knows for sure why we see such bizarre behaviors in this curve. These 22 years have been characterized by tremendous instability in oil prices, by the biggest financial crisis the world has seen since the 1920s (2008-2014), a large war in the Middle East, the Covid-19 pandemic of catastrophic proportions, and now a war in Europe of unknown repercussions.

	So, do not feel so uneasy with the Phillips curve. We just have too many shocks.

""";
  ╠═╡ =#

# ╔═╡ 564c8af7-7df7-47d5-909c-d0718e43584e
md"""
#### Exercise 7. Supporting cells (The Okun's law)
"""

# ╔═╡ 1e269aef-f564-4f6b-aaf5-f3a34e03001a
USdata = CSV.read("US_Data_May2021.csv", DataFrame);

# ╔═╡ 1f6f07ea-751b-4a47-85f8-9709f0af06d9
begin
	function hp_filter(USdata::Array{Float64,1}, lambda::Float64)
    	n = length(USdata)
    	@assert n >= 4
    	diag2 = lambda*ones(n-2)
    	diag1 = [ -2lambda; -4lambda*ones(n-3); -2lambda ]
    	diag0 = [ 1+lambda; 1+5lambda; (1+6lambda)*ones(n-4); 1+5lambda; 1+lambda ]
    	D = spdiagm(-2 => diag2, -1 => diag1, 0 => diag0, 1 => diag1, 2 => diag2)
    	D\USdata
	end
end;

# ╔═╡ d6557f28-2d14-4d42-9c6f-ad1d114595de
period7_8 = QuarterlyDate(1960, 1):Quarter(1):QuarterlyDate(2019, 4);

# ╔═╡ b5e27f26-96cc-4409-a56c-489336d906a2
begin
	GDP = log.(USdata[1:240, 2]); # We consider only the pre-COVID period (last 4 obervations dropped)
	U = (USdata[1:240, 8]); # We consider only the pre-COVID period (last 4 obervations dropped)

	# GDP
	GDP_hptrend = hp_filter(GDP, 1600.0);
	GDP_cycles = (GDP-GDP_hptrend)*100;

	# Unemployment
	U_hptrend = hp_filter(U, 1600.0);
	U_cycles = U-U_hptrend;
end;


# ╔═╡ 3da18742-5278-4fdf-8f04-30c4b4706fb2
begin
	trace7_12 = scatter(;x = Date.(period7_8), y = U, name = "U", line_color = "Blue", 
						mode = "markers+lines", line_width="0.3", marker_size ="5")
	
	trace7_13 = scatter(;x = Date.(period7_8), y = U_hptrend, name = "Trend", line_color = "Red", 
						mode = "markers+lines", line_width="0.3", marker_size ="5")

	layout7_13 = Layout(;
				height = 450,
				title_text = "Fig 8.1: US Unemployment and its Trend (1960-2019)",
				title_x = 0.5,
				xaxis_range = [Date(1959) , Date(2021)],
		
				hovermode="x",
		
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1960/04/01",
                dtick = "M120",
        				),
        		#xaxis_range = [1960, 2020],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	p7_13 = plot([trace7_12, trace7_13], layout7_13)
end

# ╔═╡ 0afd818b-856a-4000-82ac-a7e68cb48391
begin
	trace7_14 = scatter(;x = Date.(period7_8), y = U_cycles, name = "U cyclical", mode = "markers+lines", line_color = "darkblue", line_width="0.5", marker_size ="5")
	#shapes = hline([0, 0], line_color="green", line_dash="dot", line_width = "1")

	layout7_14 = Layout(; #shapes = shapes, 
				#zero_line = "true", line_width = "4",
				height = 450,
		
				title_text = "Fig 8.2: US Cyclical Unemployment (1960-2019)",
				title_x = 0.5,
				xaxis_range = [Date(1959) , Date(2021)],
		
				hovermode="x",
		
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1960/04/01",
                dtick = "M120",
        				),
        		#xaxis_range = [1960, 2020],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	p7_14 = plot([trace7_14], layout7_14)
	add_shape!(p7_14, line( x0="1959-01-01", y0=0, x1="2021-01-01", y1=0 , line=attr(color="maroon", width=2)))
	p7_14
end

# ╔═╡ 9e354338-1870-4d17-b470-2e1beda77aa1
begin
	trace7_15 = scatter(;x = Date.(period7_8), y = GDP, name = "GDP", line_color = "Blue", 
						mode = "lines", line_width="1.5")
	
	trace7_16 = scatter(;x = Date.(period7_8), y = GDP_hptrend, name = "Trend", line_color = "Red", 
						mode = "lines", line_width="1.5")

	layout7_16 = Layout(;
				height = 450,
				title_text = "Fig 8.3: GDP and its Trend (1960-2019)",
				title_x = 0.5,
		
				hovermode="x",
		
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1960/04/01",
                dtick = "M120",
        				),
        		#xaxis_range = [1960, 2020],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	p7_16 = plot([trace7_15, trace7_16], layout7_16)
end

# ╔═╡ e105541d-b332-4488-a3fe-870b9c3ca232
begin
	trace7_17 = scatter(;x = Date.(period7_8), y = GDP_cycles, name = "Output-gap", line_color = "DarkRed", 
						mode = "markers+lines", line_width="0.3", marker_size ="5")

	layout7_17 = Layout(;
				height = 450,
				title_text = "Fig 8.4: Output Gap (1960-2019)",
				title_x = 0.5,
				xaxis_range = [Date(1959) , Date(2021)],
		
				hovermode="x",
		
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1960/04/01",
                dtick = "M120",
        				),
        		#xaxis_range = [1960, 2020],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	p7_17 = plot([trace7_17], layout7_17)
	add_shape!(p7_17, line( x0="1959-01-01", y0=0, x1="2021-01-01", y1=0 , line=attr(color="RoyalBlue", width=2)))
	p7_17
end

# ╔═╡ 781d785f-987b-4eb4-aeac-1a3277d6d0ea
begin
	# linear regression using Julia's linear algebra
	X = hcat(ones(length(GDP_cycles)),GDP_cycles)
	Y = U_cycles
	intercept,slope = inv(X'* X)*(X'* Y)
end;

# ╔═╡ 8fef9bcd-2cc7-4ff3-87ed-9d0fba75dd1b
begin
	gap = -5:0.01:4
	Okun = slope .* gap
	Print(slope)
end;

# ╔═╡ 1eabd160-ae3c-4c97-9c0e-ae0b95fdf8bb
begin
	
	trace7_20 = scatter(; x = GDP_cycles , y = U_cycles, mode="markers+lines" , text=period7_8,
					marker_symbol="circle", marker_size="7", line_width=0.0, 
					marker_color="Blue", name  = "Points")

	trace7_31 = scatter(; x=gap, y = Okun,
			line_width= 3, 
            line_dash="solid", 
            line_color="Red",
            #line_color="Pink",
            name="Okun's Law")
	
	
	layout7_20 = Layout(;
					height = 450,
					title_text = "Fig 8.5: The Okun's Law for the USA (1960-2019)",
					title_x = 0.5,
        			yaxis_title = "Cyclical unemployment",
		  			#xaxis_range = [1960, 2020],
        			xaxis_title = "Output-gap",
        			#yaxis_range=[-2, 2], 
        			titlefont_size=18)

	p7_20 = plot([trace7_20 , trace7_31 ], layout7_20)
	#addtraces(p7_10, t22)
	
end

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
CSV = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
Dates = "ade2ca70-3891-5945-98fb-dc099432e06a"
HypertextLiteral = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
PeriodicalDates = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
PlutoPlotly = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
StatsBase = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"

[compat]
CSV = "~0.10.9"
DataFrames = "~1.7.0"
HypertextLiteral = "~0.9.5"
PeriodicalDates = "~2.0.0"
PlutoPlotly = "~0.6.1"
PlutoUI = "~0.7.50"
StatsBase = "~0.34.3"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.11.3"
manifest_format = "2.0"
project_hash = "58531e5de0291cb7cc309141f9652ed50c2ba105"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "6e1d2a35f2f90a4bc7c2ed98079b2ba09c35b83a"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.3.2"

[[deps.AliasTables]]
deps = ["PtrArrays", "Random"]
git-tree-sha1 = "9876e1e164b144ca45e9e3198d0b689cadfed9ff"
uuid = "66dad0bd-aa9a-41b7-9441-69ab47430ed8"
version = "1.1.3"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.2"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"
version = "1.11.0"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"
version = "1.11.0"

[[deps.CSV]]
deps = ["CodecZlib", "Dates", "FilePathsBase", "InlineStrings", "Mmap", "Parsers", "PooledArrays", "PrecompileTools", "SentinelArrays", "Tables", "Unicode", "WeakRefStrings", "WorkerUtilities"]
git-tree-sha1 = "deddd8725e5e1cc49ee205a1964256043720a6c3"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.10.15"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "962834c22b66e32aa10f7611c08c8ca4e20749a9"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.8"

[[deps.ColorSchemes]]
deps = ["ColorTypes", "ColorVectorSpace", "Colors", "FixedPointNumbers", "PrecompileTools", "Random"]
git-tree-sha1 = "403f2d8e209681fcbd9468a8514efff3ea08452e"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.29.0"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "b10d0b65641d57b8b4d5e234446582de5047050d"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.5"

[[deps.ColorVectorSpace]]
deps = ["ColorTypes", "FixedPointNumbers", "LinearAlgebra", "Requires", "Statistics", "TensorCore"]
git-tree-sha1 = "a1f44953f2382ebb937d60dafbe2deea4bd23249"
uuid = "c3611d14-8923-5661-9e6a-0046d554d3a4"
version = "0.10.0"

    [deps.ColorVectorSpace.extensions]
    SpecialFunctionsExt = "SpecialFunctions"

    [deps.ColorVectorSpace.weakdeps]
    SpecialFunctions = "276daf66-3868-5448-9aa4-cd146d93841b"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "362a287c3aa50601b0bc359053d5c2468f0e7ce0"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.11"

[[deps.Compat]]
deps = ["TOML", "UUIDs"]
git-tree-sha1 = "8ae8d32e09f0dcf42a36b90d4e17f5dd2e4c4215"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.16.0"
weakdeps = ["Dates", "LinearAlgebra"]

    [deps.Compat.extensions]
    CompatLinearAlgebraExt = "LinearAlgebra"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "1.1.1+0"

[[deps.Crayons]]
git-tree-sha1 = "249fe38abf76d48563e2f4556bebd215aa317e15"
uuid = "a8cc5b0e-0ffa-5ad4-8c14-923d3ee1735f"
version = "4.1.1"

[[deps.DataAPI]]
git-tree-sha1 = "abe83f3a2f1b857aac70ef8b269080af17764bbe"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.16.0"

[[deps.DataFrames]]
deps = ["Compat", "DataAPI", "DataStructures", "Future", "InlineStrings", "InvertedIndices", "IteratorInterfaceExtensions", "LinearAlgebra", "Markdown", "Missings", "PooledArrays", "PrecompileTools", "PrettyTables", "Printf", "Random", "Reexport", "SentinelArrays", "SortingAlgorithms", "Statistics", "TableTraits", "Tables", "Unicode"]
git-tree-sha1 = "fb61b4812c49343d7ef0b533ba982c46021938a6"
uuid = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
version = "1.7.0"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "1d0a14036acb104d9e89698bd408f63ab58cdc82"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.20"

[[deps.DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"
version = "1.11.0"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
git-tree-sha1 = "9e2f36d3c96a820c678f2f1f1782582fcf685bae"
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"
version = "1.9.1"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "2fb1e02f2b635d0845df5d7c167fec4dd739b00d"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.3"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.FilePathsBase]]
deps = ["Compat", "Dates"]
git-tree-sha1 = "2ec417fc319faa2d768621085cc1feebbdee686b"
uuid = "48062228-2e41-5def-b9a4-89aafe57970f"
version = "0.9.23"
weakdeps = ["Mmap", "Test"]

    [deps.FilePathsBase.extensions]
    FilePathsBaseMmapExt = "Mmap"
    FilePathsBaseTestExt = "Test"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"
version = "1.11.0"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "05882d6995ae5c12bb5f36dd2ed3f61c98cbb172"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.5"

[[deps.Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"
version = "1.11.0"

[[deps.HashArrayMappedTries]]
git-tree-sha1 = "2eaa69a7cab70a52b9687c8bf950a5a93ec895ae"
uuid = "076d061b-32b6-4027-95e0-9a2c6f6d7e74"
version = "0.2.0"

[[deps.Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "179267cfa5e712760cd43dcae385d7ea90cc25a4"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.5"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "7134810b1afce04bbc1045ca1985fbe81ce17653"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.5"

[[deps.IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "b6d6bfdd7ce25b0f9b2f6b3dd56b2673a66c8770"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.5"

[[deps.InlineStrings]]
git-tree-sha1 = "6a9fde685a7ac1eb3495f8e812c5a7c3711c2d5e"
uuid = "842dd82b-1e85-43dc-bf29-5d0ee9dffc48"
version = "1.4.3"

    [deps.InlineStrings.extensions]
    ArrowTypesExt = "ArrowTypes"
    ParsersExt = "Parsers"

    [deps.InlineStrings.weakdeps]
    ArrowTypes = "31f734f8-188a-4ce0-8406-c8a06bd891cd"
    Parsers = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"
version = "1.11.0"

[[deps.InvertedIndices]]
git-tree-sha1 = "6da3c4316095de0f5ee2ebd875df8721e7e0bdbe"
uuid = "41ab1584-1d38-5bbf-9106-f11c6c58b48f"
version = "1.3.1"

[[deps.IrrationalConstants]]
git-tree-sha1 = "e2222959fbc6c19554dc15174c81bf7bf3aa691c"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.2.4"

[[deps.IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "31e996f0a15c7b280ba9f76636b3ff9e2ae58c9a"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.4"

[[deps.LaTeXStrings]]
git-tree-sha1 = "dda21b8cbd6a6c40d9d02a73230f9d70fed6918c"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.4.0"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.4"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "8.6.0+0"

[[deps.LibGit2]]
deps = ["Base64", "LibGit2_jll", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"
version = "1.11.0"

[[deps.LibGit2_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll"]
uuid = "e37daf67-58a4-590a-8e99-b0245dd2ffc5"
version = "1.7.2+0"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.11.0+1"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"
version = "1.11.0"

[[deps.LinearAlgebra]]
deps = ["Libdl", "OpenBLAS_jll", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
version = "1.11.0"

[[deps.LogExpFunctions]]
deps = ["DocStringExtensions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "13ca9e2586b89836fd20cccf56e57e2b9ae7f38f"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.29"

    [deps.LogExpFunctions.extensions]
    LogExpFunctionsChainRulesCoreExt = "ChainRulesCore"
    LogExpFunctionsChangesOfVariablesExt = "ChangesOfVariables"
    LogExpFunctionsInverseFunctionsExt = "InverseFunctions"

    [deps.LogExpFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    ChangesOfVariables = "9e997f8a-9a97-42d5-a9f1-ce6bfc15e2c0"
    InverseFunctions = "3587e190-3f89-42d0-90ee-14403ec27112"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"
version = "1.11.0"

[[deps.MIMEs]]
git-tree-sha1 = "1833212fd6f580c20d4291da9c1b4e8a655b128e"
uuid = "6c6e2e6c-3030-632d-7369-2d6c69616d65"
version = "1.0.0"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"
version = "1.11.0"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.6+0"

[[deps.Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "ec4f7fbeab05d7747bdf98eb74d130a2a2ed298d"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.2.0"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"
version = "1.11.0"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2023.12.12"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.27+1"

[[deps.OrderedCollections]]
git-tree-sha1 = "cc4054e898b852042d7b503313f7ad03de99c3dd"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.8.0"

[[deps.Parameters]]
deps = ["OrderedCollections", "UnPack"]
git-tree-sha1 = "34c0e9ad262e5f7fc75b10a9952ca7692cfc5fbe"
uuid = "d96e819e-fc66-5662-9728-84c9c7592b0a"
version = "0.12.3"

[[deps.Parsers]]
deps = ["Dates", "PrecompileTools", "UUIDs"]
git-tree-sha1 = "8489905bcdbcfac64d1daa51ca07c0d8f0283821"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.8.1"

[[deps.PeriodicalDates]]
deps = ["Dates", "Printf", "RecipesBase"]
git-tree-sha1 = "e616941f8093e50a373e7d51875143213f82eca4"
uuid = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
version = "2.0.0"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "FileWatching", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "Random", "SHA", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.11.0"
weakdeps = ["REPL"]

    [deps.Pkg.extensions]
    REPLExt = "REPL"

[[deps.PlotlyBase]]
deps = ["ColorSchemes", "Colors", "Dates", "DelimitedFiles", "DocStringExtensions", "JSON", "LaTeXStrings", "Logging", "Parameters", "Pkg", "REPL", "Requires", "Statistics", "UUIDs"]
git-tree-sha1 = "90af5c9238c1b3b25421f1fdfffd1e8fca7a7133"
uuid = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
version = "0.8.20"

    [deps.PlotlyBase.extensions]
    DataFramesExt = "DataFrames"
    DistributionsExt = "Distributions"
    IJuliaExt = "IJulia"
    JSON3Ext = "JSON3"

    [deps.PlotlyBase.weakdeps]
    DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
    Distributions = "31c24e10-a181-5473-b8eb-7969acd0382f"
    IJulia = "7073ff75-c697-5162-941a-fcdaad2a7d2a"
    JSON3 = "0f8b85d8-7281-11e9-16c2-39a750bddbf1"

[[deps.PlutoPlotly]]
deps = ["AbstractPlutoDingetjes", "Artifacts", "ColorSchemes", "Colors", "Dates", "Downloads", "HypertextLiteral", "InteractiveUtils", "LaTeXStrings", "Markdown", "Pkg", "PlotlyBase", "PrecompileTools", "Reexport", "ScopedValues", "Scratch", "TOML"]
git-tree-sha1 = "9ebe25fc4703d4112cc418834d5e4c9a4b29087d"
uuid = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
version = "0.6.2"

    [deps.PlutoPlotly.extensions]
    PlotlyKaleidoExt = "PlotlyKaleido"
    UnitfulExt = "Unitful"

    [deps.PlutoPlotly.weakdeps]
    PlotlyKaleido = "f2990250-8cf9-495f-b13a-cce12b45703c"
    Unitful = "1986cc42-f94f-5a68-af5c-568840ba703d"

[[deps.PlutoUI]]
deps = ["AbstractPlutoDingetjes", "Base64", "ColorTypes", "Dates", "FixedPointNumbers", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "MIMEs", "Markdown", "Random", "Reexport", "URIs", "UUIDs"]
git-tree-sha1 = "7e71a55b87222942f0f9337be62e26b1f103d3e4"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.61"

[[deps.PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "36d8b4b899628fb92c2749eb488d884a926614d3"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.4.3"

[[deps.PrecompileTools]]
deps = ["Preferences"]
git-tree-sha1 = "5aa36f7049a63a1528fe8f7c3f2113413ffd4e1f"
uuid = "aea7be01-6a6a-4083-8856-8a6e6704d82a"
version = "1.2.1"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "9306f6085165d270f7e3db02af26a400d580f5c6"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.4.3"

[[deps.PrettyTables]]
deps = ["Crayons", "LaTeXStrings", "Markdown", "PrecompileTools", "Printf", "Reexport", "StringManipulation", "Tables"]
git-tree-sha1 = "1101cd475833706e4d0e7b122218257178f48f34"
uuid = "08abe8d2-0d0c-5749-adfa-8a2ac140af0d"
version = "2.4.0"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"
version = "1.11.0"

[[deps.PtrArrays]]
git-tree-sha1 = "1d36ef11a9aaf1e8b74dacc6a731dd1de8fd493d"
uuid = "43287f4e-b6f4-7ad1-bb20-aadabca52c3d"
version = "1.3.0"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "StyledStrings", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"
version = "1.11.0"

[[deps.Random]]
deps = ["SHA"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"
version = "1.11.0"

[[deps.RecipesBase]]
deps = ["PrecompileTools"]
git-tree-sha1 = "5c3d09cc4f31f5fc6af001c250bf1278733100ff"
uuid = "3cdcf5f2-1ef4-517c-9805-6587b60abb01"
version = "1.3.4"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "838a3a4188e2ded87a4f9f184b4b0d78a1e91cb7"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.0"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.ScopedValues]]
deps = ["HashArrayMappedTries", "Logging"]
git-tree-sha1 = "1147f140b4c8ddab224c94efa9569fc23d63ab44"
uuid = "7e506255-f358-4e82-b7e4-beb19740aa63"
version = "1.3.0"

[[deps.Scratch]]
deps = ["Dates"]
git-tree-sha1 = "3bac05bc7e74a75fd9cba4295cde4045d9fe2386"
uuid = "6c6a2e73-6563-6170-7368-637461726353"
version = "1.2.1"

[[deps.SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "712fb0231ee6f9120e005ccd56297abbc053e7e0"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.4.8"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"
version = "1.11.0"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"
version = "1.11.0"

[[deps.SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "66e0a8e672a0bdfca2c3f5937efb8538b9ddc085"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.2.1"

[[deps.SparseArrays]]
deps = ["Libdl", "LinearAlgebra", "Random", "Serialization", "SuiteSparse_jll"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
version = "1.11.0"

[[deps.Statistics]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "ae3bb1eb3bba077cd276bc5cfc337cc65c3075c0"
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"
version = "1.11.1"
weakdeps = ["SparseArrays"]

    [deps.Statistics.extensions]
    SparseArraysExt = ["SparseArrays"]

[[deps.StatsAPI]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1ff449ad350c9c4cbc756624d6f8a8c3ef56d3ed"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.7.0"

[[deps.StatsBase]]
deps = ["AliasTables", "DataAPI", "DataStructures", "LinearAlgebra", "LogExpFunctions", "Missings", "Printf", "Random", "SortingAlgorithms", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "29321314c920c26684834965ec2ce0dacc9cf8e5"
uuid = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"
version = "0.34.4"

[[deps.StringManipulation]]
deps = ["PrecompileTools"]
git-tree-sha1 = "725421ae8e530ec29bcbdddbe91ff8053421d023"
uuid = "892a3eda-7b42-436c-8928-eab12a02cf0e"
version = "0.4.1"

[[deps.StyledStrings]]
uuid = "f489334b-da3d-4c2e-b8f0-e476e12c162b"
version = "1.11.0"

[[deps.SuiteSparse_jll]]
deps = ["Artifacts", "Libdl", "libblastrampoline_jll"]
uuid = "bea87d4a-7f5b-5778-9afe-8cc45184846c"
version = "7.7.0+0"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[deps.TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[deps.Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "OrderedCollections", "TableTraits"]
git-tree-sha1 = "598cd7c1f68d1e205689b1c2fe65a9f85846f297"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.12.0"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.TensorCore]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1feb45f88d133a655e001435632f019a9a1bcdb6"
uuid = "62fd8b95-f654-4bbd-a8a5-9c27f68ccd50"
version = "0.1.1"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"
version = "1.11.0"

[[deps.TranscodingStreams]]
git-tree-sha1 = "0c45878dcfdcfa8480052b6ab162cdd138781742"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.11.3"

[[deps.Tricks]]
git-tree-sha1 = "6cae795a5a9313bbb4f60683f7263318fc7d1505"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.10"

[[deps.URIs]]
git-tree-sha1 = "67db6cc7b3821e19ebe75791a9dd19c9b1188f2b"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.5.1"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"
version = "1.11.0"

[[deps.UnPack]]
git-tree-sha1 = "387c1f73762231e86e0c9c5443ce3b4a0a9a0c2b"
uuid = "3a884ed6-31ef-47d7-9d2a-63182c4928ed"
version = "1.0.2"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"
version = "1.11.0"

[[deps.WeakRefStrings]]
deps = ["DataAPI", "InlineStrings", "Parsers"]
git-tree-sha1 = "b1be2855ed9ed8eac54e5caff2afcdb442d52c23"
uuid = "ea10d353-3f73-51f8-a26c-33c1cb351aa5"
version = "1.4.2"

[[deps.WorkerUtilities]]
git-tree-sha1 = "cd1659ba0d57b71a464a29e64dbc67cfe83d54e7"
uuid = "76eceee3-57b5-4d4a-8e66-0e911cebbf60"
version = "1.6.1"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.13+1"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.11.0+0"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.59.0+0"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+2"
"""

# ╔═╡ Cell order:
# ╟─fd79048d-2700-4f42-ac1e-5cb4df35e5ea
# ╟─943edb36-acc8-4b2a-94ab-c544f9eb279b
# ╟─1245aef9-18ce-42f5-af82-c2e6cce70798
# ╟─6b04fe4f-5078-4cf0-bad6-6ea01d4415b0
# ╟─4618a663-f77e-4ccc-9b00-e84dbd5beb34
# ╠═a145e2fe-a903-11eb-160f-df7ea83fa3e6
# ╟─ec4b312d-c945-4d24-ad88-6fa15ed6f0f1
# ╟─388f775e-c93a-4980-affe-5ba2521616a3
# ╟─a726c21b-578e-4167-94b0-4f364ac2be12
# ╟─fcea4d0b-a7c9-4d0b-b0da-1a903ed0bc8d
# ╟─96090bb8-be09-48ac-8a8c-43c7f5f01808
# ╟─a3a7ca81-c5ad-4ee5-9cc9-65e017c45bb3
# ╟─0fed5c77-3cd8-4a32-a2d0-c5bd9cd4ba0e
# ╟─5a8c4fea-e7e5-4713-808a-1e6ff167d6fa
# ╟─e1b4af55-b12d-4cfb-b2a6-9ef8e5311a70
# ╟─9bd58808-8f91-49f7-92a1-19bf05e7accc
# ╟─eb7f7dff-8f31-4793-bcdd-1e1c47363abb
# ╟─8022fa16-02c3-4481-b7ba-2fc1ded4f344
# ╟─0d2d4c66-b49c-46c0-8ed0-3906db9c8deb
# ╟─27746311-ce1b-4b68-9bc2-8dcf1306a593
# ╟─84238089-0c10-4c03-a494-073e4d317673
# ╟─5ea54f91-bd86-4454-b6ef-3a97a0105ec2
# ╟─4dbc64ac-79c2-4308-87c4-5e57e3c1e7a2
# ╟─241b7098-b3f3-4e36-81d4-dafa822c49c9
# ╟─789b9f2b-4d0c-40b9-9ca4-809abbdfcba2
# ╟─5ed4a452-13c4-49b1-8df8-f60b7ce3d905
# ╟─3ede19ba-d5c1-4313-bd70-2d0ba03fa02c
# ╟─31a42c8d-41db-4fdf-9af8-3740477294d1
# ╟─dcd13560-0f73-4aaa-b59a-b7961d8cdc63
# ╟─7f67d656-bb55-4097-ba49-10140f9b3502
# ╟─beb52783-708e-4782-b271-643f2a4f3297
# ╟─66463532-449b-4cac-8083-8938786a193b
# ╟─466472a7-72f5-4ac2-9688-fc3fcdf3c09f
# ╟─28e51405-60fb-496c-b3cd-5aa256243b8e
# ╟─d1d2aaf7-d548-4b0d-9668-28ff262a8734
# ╟─57787a16-2568-4fcb-94cf-7f5533facf05
# ╟─a91c2b80-3215-46ac-938c-ef98e4dde437
# ╟─95dce14a-b5f9-4dd8-af46-415e71b425fb
# ╟─62d8a5a1-eab1-49ae-8947-b5299d1217c5
# ╟─6949d31b-c934-42f8-a53e-8fd1e068fff4
# ╟─95b7bd36-e0fc-4464-8bbe-9e9f491dae64
# ╟─42f4b471-f8fe-4c75-a2dd-2383a4c683f2
# ╟─21d79f5f-f03a-4d63-8f79-d8868bd9234d
# ╟─bf93b0c6-ff58-4603-b0ac-57f802e473b1
# ╟─763bc186-dae2-479c-8195-6585b46733be
# ╟─63ee9ae5-d84e-4b8d-bf65-88be736e3843
# ╟─f0196aef-6843-4f69-b387-ca5deec33604
# ╟─10ccbe74-dff0-4f84-9b8d-3563559be3ba
# ╟─d333ae08-ecb5-454f-a963-2dc33583d21c
# ╟─d1cd84a3-10dd-4399-982b-52d149b9094c
# ╟─add07024-b7eb-445d-b4f4-6a6757c2ecd3
# ╟─bfb890be-fba4-4d55-9e39-a52d40a5adad
# ╟─f1b510d9-e9b2-4db6-a738-ac8536f392f7
# ╟─8440209f-17c5-4ed7-9a98-8a5a2905a694
# ╟─7478d0ce-4ae1-4949-9ae4-2ffa334454f9
# ╟─31b7a1b6-6849-41cc-92d0-f9c717bd3dda
# ╟─fdec5907-d925-4ae2-9298-fd14f3213139
# ╟─78475df7-5d81-410c-97bb-9ce06e897a7e
# ╟─1aadb49f-b6f8-4b56-b6ba-c529d97ecd3f
# ╟─7bc32249-45ff-42bf-8d39-ec1a7670fbdd
# ╟─04b850b4-e659-4f1c-82be-ba912bad03af
# ╟─6ddb604e-3455-49d4-9298-ada4bf0227fc
# ╟─24f2f06e-05fe-4b12-b238-d660f449ee9b
# ╟─e9ac0a44-22ec-40d6-9fc1-2d9c8949455e
# ╟─f8214ab3-f5f1-4d1d-9e2f-8a2fba5a929c
# ╟─c3fdfb74-1e85-487f-8715-0b083004aa03
# ╟─793b3108-76c1-4274-9861-c9ee948eca88
# ╟─a17fc5ea-9598-40aa-be2f-864340b424a3
# ╟─06a65cea-8587-4390-9f62-0adda6b8a84e
# ╟─24f78b7c-7bda-4ea1-b593-bdacb94e8c4d
# ╟─bb5398f5-516c-4954-8cc4-b0e1c1b6502c
# ╟─86f54c54-58a5-4faf-9688-baddaa032ed2
# ╟─c0bb7086-533d-4c95-850a-b445b877d8c8
# ╟─f261756b-856c-429f-93df-af0980e535b2
# ╟─9d8be89f-9ebe-45f0-b4b3-a74f7e3ee9de
# ╟─a66fed79-59fe-4993-81c2-acbff747332a
# ╟─5fa00216-1464-4f2d-80cf-4d333d9dced3
# ╟─3cf86819-09f0-43d3-aff7-494be276bf0f
# ╟─ccc8aa66-cb8d-4bbe-8457-f496f7bff47d
# ╟─3da18742-5278-4fdf-8f04-30c4b4706fb2
# ╟─0afd818b-856a-4000-82ac-a7e68cb48391
# ╟─208bbede-ad8d-45f0-8524-43948733ffb8
# ╟─dc5bfa8c-7247-44fc-8837-a7fa32c36d54
# ╟─d040f3bb-7ea9-4000-813a-6e1f4e0163cb
# ╟─9e354338-1870-4d17-b470-2e1beda77aa1
# ╟─e105541d-b332-4488-a3fe-870b9c3ca232
# ╟─ea2e3d0e-c933-4d67-ac24-3ba9bcf6fc8a
# ╟─2ab10860-0671-4e3c-a7ea-dff6ae80c528
# ╟─1eabd160-ae3c-4c97-9c0e-ae0b95fdf8bb
# ╟─cc770177-1d7a-4f6e-a930-c805b3bfee6d
# ╟─4d7901da-39a9-4f24-830f-0ecd945d41dd
# ╟─03706c92-80c5-41d7-bbc3-4e68d1873e77
# ╟─709997bf-ae42-49e8-8224-b9c2aae30bdb
# ╟─df7eac78-5c6c-4307-b2ae-ad2331d99c4a
# ╟─b0c297b8-b49c-4b45-b294-1f865d6ca2d2
# ╠═7ee61691-f0ae-40c9-8d42-2ca45a3317d2
# ╟─926e7520-8c1e-4dc2-a9a5-0bafcd13bf47
# ╟─66b70b7b-c792-44b7-8045-68ad836d1616
# ╟─1b35a651-08b3-4598-b830-b5bc9d73df34
# ╟─506e4667-627c-4808-83be-ef9fe2211f87
# ╟─c3fc8216-c016-4346-b7bd-1c4b421f2b37
# ╟─051bf8f6-d581-4a51-955f-02d78446c0f1
# ╟─df9417d2-ebe1-498c-b299-f2b8fdee1084
# ╠═b28d588a-b4ff-444b-8b16-3cd05110ad1d
# ╟─7087371c-98d3-4e8b-a0e8-47c0631ee59e
# ╟─90c6188b-4356-415a-94ce-dc978d3b15ac
# ╟─03c7091d-7830-4fad-b444-413454736d5b
# ╟─0735c5c2-f79c-4424-8fac-e060280d904c
# ╟─239ccedb-f91d-4eb1-9a69-588bf4ff4ebc
# ╟─8fc49bd4-075d-4e3f-b50b-5f381c09ff10
# ╟─8430162f-4f13-4525-9260-027d84222808
# ╟─ae1c8887-678f-4485-a492-a1b06e7aaefa
# ╟─db2990be-db70-456f-96c4-9de707bfcfbd
# ╟─4c73146a-3e09-4125-8cfd-5665206d68b6
# ╟─7d77dff4-dbe2-4ee0-9be3-fbdaee9edaa8
# ╟─f3b6fae7-44de-496d-98a3-11f416e62f20
# ╟─34161592-b86c-428e-ad0f-6c7b6c8c1be5
# ╟─aafdc1f7-7f2a-454c-b3da-9c9c68aed107
# ╟─b210c2e6-c9a5-44d8-a01c-c9702da0606d
# ╟─aea3a28e-c408-4a6d-ac6a-1507ed4e4d1b
# ╟─c1a3d11b-881c-4c74-ac1d-e794562349e0
# ╟─778b7504-2f80-480e-bf43-cdbea4be4514
# ╟─854c4538-8d47-41f6-ba61-edfd81f632d1
# ╟─5262ff53-633c-46c9-b812-f31d8c90a873
# ╟─9cd31e46-2efd-4cb7-a1fb-833fb0214d84
# ╟─9fa87191-7c90-4738-a45a-acd929c8bd1b
# ╟─b9c0b1d6-2289-472d-92ce-576c79f73268
# ╟─f208e537-179c-444f-ba47-96b15ce74855
# ╟─9dcec3b2-c378-4814-afed-6997be743699
# ╟─d68f00b3-1e31-44bb-b48c-d53580c8893d
# ╟─d9aaf9ef-c3aa-4c9f-9762-e1d818ed0edd
# ╟─7ab54cda-93a0-4969-870d-4dc15879a8a7
# ╟─af29458d-2ee7-406b-bbc9-8b9a2e94fa39
# ╟─8205aa02-e1a1-433a-a4ff-7491dd07faa7
# ╟─51afa448-613b-4a9f-8af8-a0b02abba89c
# ╟─518caed6-cb92-4410-bc0a-7a5c7bdc155e
# ╟─f613e456-403b-4bff-bd65-315b1be5eb45
# ╟─f5ae46d3-3f3d-4231-ad25-4333af5949e3
# ╟─2625f138-a5e6-42a3-add7-c648b7334b57
# ╟─5e8b862d-031c-48e4-8abd-8532637d04a2
# ╟─cb1e02e1-9baf-4e47-b5bf-ecf472ad6fde
# ╟─1350111b-499c-4079-aa9d-103c081e0e74
# ╟─ba40d0ff-992c-44de-b1a2-7ffe030bb3fd
# ╟─aa86d58a-214d-4ef3-a2a3-86b78af55967
# ╟─af88f312-87dd-4a1e-913d-31ba12f06b45
# ╟─f4d5a7be-361c-4a52-b501-b363548e42ee
# ╟─564c8af7-7df7-47d5-909c-d0718e43584e
# ╟─1e269aef-f564-4f6b-aaf5-f3a34e03001a
# ╟─1f6f07ea-751b-4a47-85f8-9709f0af06d9
# ╟─d6557f28-2d14-4d42-9c6f-ad1d114595de
# ╟─781d785f-987b-4eb4-aeac-1a3277d6d0ea
# ╟─b5e27f26-96cc-4409-a56c-489336d906a2
# ╟─8fef9bcd-2cc7-4ff3-87ed-9d0fba75dd1b
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
